"""
    pyap.exceptions
    ~~~~~~~~~~~~~~~~

    This module contains address parser exceptions.

    :copyright: (c) 2015 by Vladimir Goncharov.
    :license: MIT, see LICENSE for more details.
"""


class AddressParserException(Exception):
    pass


class NoCountrySelected(AddressParserException):
    """No country selected during module initialization"""

    def __init__(self, message: str, errors: str):
        super().__init__(message)
        self.errors = errors


class CountryDetectionMissing(AddressParserException):
    """Country-specific address detection rules were not found"""

    def __init__(self, message: str, errors: str):
        super().__init__(message)
        self.errors = errors



"""
pyap.utils
~~~~~~~~~~~~~~~~

This module provides some utility functions.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""
import re
from re import Match, RegexFlag
from typing import List, Optional

DEFAULT_FLAGS = re.VERBOSE | re.UNICODE


def match(regex: str, string: str, flags: RegexFlag = DEFAULT_FLAGS) -> Optional[Match]:
    """Utility function for re.match"""
    return re.match(regex, string, flags=flags)


def findall(
    regex: str, string: str, flags: RegexFlag = DEFAULT_FLAGS
) -> List[Optional[Match]]:
    """Utility function for re.findall"""
    return re.findall(regex, string, flags=flags)


def finditer(regex: str, string: str, flags: RegexFlag = DEFAULT_FLAGS) -> List[Match]:
    """Utility function for re.finditer"""
    return list(re.finditer(regex, string, flags=flags))



"""
    pyap.address
    ~~~~~~~~~~~~~~~~

    Contains class for constructing Address object which holds information
    about address and its components.

    :copyright: (c) 2015 by Vladimir Goncharov.
    :license: MIT, see LICENSE for more details.
"""
from typing import Any, Optional, Union

from pydantic import BaseModel, field_validator


class Address(BaseModel):
    building_id: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    country_id: Optional[str] = None
    floor: Optional[str] = None
    full_address: str
    full_street: Optional[str] = None
    match_end: Optional[Union[int, str]] = None
    match_start: Optional[Union[int, str]] = None
    occupancy: Optional[str] = None
    postal_code: Optional[str] = None
    region1: Optional[str] = None
    route_id: Optional[str] = None
    state: Optional[str] = None
    street: Optional[str] = None
    street_name: Optional[str] = None
    street_number: Optional[str] = None
    street_type: Optional[str] = None

    @field_validator("*", mode="before")
    @classmethod
    def strip_chars(cls, v: Any) -> Any:
        if isinstance(v, str):
            return v.strip(" ,;:")
        if v:
            return v

    def __str__(self) -> str:
        # Address object is represented as textual address
        address = ""
        try:
            address = self.full_address
        except AttributeError:
            pass

        return address



"""
    pyap.api
    ~~~~~~~~~~~~~~~~

    This module contains address parser API functions.

    :copyright: (c) 2015 by Vladimir Goncharov.
    :license: MIT, see LICENSE for more details.
"""
from typing import Any, List, Optional

from pyap_beauhurst.address import Address

from . import parser


def parse(some_text: str, **kwargs: Any) -> List[Optional[Address]]:
    """Creates request to AddressParser
    and returns list of Address objects
    """
    ap = parser.AddressParser(**kwargs)
    return ap.parse(some_text)
